@@ -1,15 +1,10 @@
-import typing
 import asyncio
+import typing
 from collections.abc import Mapping
 
+from rpcpy.datastructures import URL, Headers, MutableHeaders
 from rpcpy.types import Message, Receive, Scope, Send
 from rpcpy.utils import cached_property
-from rpcpy.datastructures import (
-    URL,
-    Headers,
-    MutableHeaders,
-)
-
 
 __all__ = ["Request", "Response", "EventResponse"]
 
@@ -19,11 +14,11 @@ class ClientDisconnect(Exception):
 
 
 async def empty_receive() -> Message:
-    raise RuntimeError("Receive channel has not been made available")
+    raise NotImplementedError("Receive channel has not been made available")
 
 
 async def empty_send(message: Message) -> None:
-    raise RuntimeError("Send channel has not been made available")
+    raise NotImplementedError("Send channel has not been made available")
 
 
 class Request(Mapping):
@@ -179,7 +174,7 @@ class EventResponse(Response):
     ) -> None:
 
         _headers = {"Cache-Control": "no-cache", "Connection": "keep-alive"}
-        if headers:
+        if headers is not None:
             _headers.update(headers)
         super().__init__(None, status_code, _headers)
         self.generator = generator
