@@ -1,18 +1,11 @@
-import tempfile
 import typing
-import asyncio
+from itertools import chain
 from urllib.parse import SplitResult, parse_qsl, urlencode, urlsplit
 
 from rpcpy.types import Environ, Scope
 from rpcpy.utils import cached_property
 
-__all__ = [
-    "FormData",
-    "MutableHeaders",
-    "Headers",
-    "UploadFile",
-    "URL",
-]
+__all__ = ["MutableHeaders", "Headers", "URL"]
 
 
 class URL:
@@ -104,10 +97,6 @@ class URL:
     def port(self) -> typing.Optional[int]:
         return self.components.port
 
-    @property
-    def is_secure(self) -> bool:
-        return self.scheme in ("https", "wss")
-
     def replace(self, **kwargs: typing.Any) -> "URL":
         if (
             "username" in kwargs
@@ -144,9 +133,7 @@ class URL:
         query = urlencode([(str(key), str(value)) for key, value in kwargs.items()])
         return self.replace(query=query)
 
-    def remove_query_params(
-        self, keys: typing.Union[str, typing.Sequence[str]]
-    ) -> "URL":
+    def remove_query_params(self, keys: typing.Union[str, typing.Sequence[str]]) -> "URL":
         if isinstance(keys, str):
             keys = [keys]
         params = MultiDict(parse_qsl(self.query, keep_blank_values=True))
@@ -200,9 +187,7 @@ class ImmutableMultiDict(typing.Mapping):
             value = typing.cast(typing.Mapping, value)
             _items = list(value.items())
         else:
-            value = typing.cast(
-                typing.List[typing.Tuple[typing.Any, typing.Any]], value
-            )
+            value = typing.cast(typing.List[typing.Tuple[typing.Any, typing.Any]], value)
             _items = list(value)
 
         self._dict = {k: v for k, v in _items}
@@ -311,86 +296,6 @@ class MultiDict(ImmutableMultiDict):
         self._dict.update(value)
 
 
-class UploadFile:
-    """
-    An uploaded file included as part of the request data.
-    """
-
-    spool_max_size = 1024 * 1024
-
-    def __init__(self, filename: str, content_type: str = "") -> None:
-        self.filename = filename
-        self.content_type = content_type
-        self.file = tempfile.SpooledTemporaryFile(max_size=self.spool_max_size)
-
-    @property
-    def in_memory(self) -> bool:
-        rolled_to_disk = getattr(self.file, "_rolled", True)
-        return not rolled_to_disk
-
-    def write(self, data: bytes) -> None:
-        self.file.write(data)
-
-    async def awrite(self, data: bytes) -> None:
-        if self.in_memory:
-            self.write(data)  # type: ignore
-        else:
-            await asyncio.get_event_loop().run_in_executor(None, self.write, data)
-
-    def read(self, size: int = -1) -> bytes:
-        return self.file.read(size)
-
-    async def aread(self, size: int = -1) -> bytes:
-        if self.in_memory:
-            return self.read(size)
-        return await asyncio.get_event_loop().run_in_executor(None, self.read, size)
-
-    def seek(self, offset: int) -> None:
-        self.file.seek(offset)
-
-    async def aseek(self, offset: int) -> None:
-        if self.in_memory:
-            self.seek(offset)
-        else:
-            await asyncio.get_event_loop().run_in_executor(None, self.seek, offset)
-
-    def close(self) -> None:
-        self.file.close()
-
-    async def aclose(self) -> None:
-        if self.in_memory:
-            self.close()
-        else:
-            await asyncio.get_event_loop().run_in_executor(None, self.close)
-
-
-class FormData(ImmutableMultiDict):
-    """
-    An immutable multidict, containing both file uploads and text input.
-    """
-
-    def __init__(
-        self,
-        *args: typing.Union[
-            "FormData",
-            typing.Mapping[str, typing.Union[str, UploadFile]],
-            typing.List[typing.Tuple[str, typing.Union[str, UploadFile]]],
-        ],
-        **kwargs: typing.Union[str, UploadFile],
-    ) -> None:
-        super().__init__(*args, **kwargs)
-
-    def close(self) -> None:
-        for key, value in self.multi_items():
-            if isinstance(value, UploadFile):
-                value.close()
-
-    async def aclose(self) -> None:
-        for key, value in self.multi_items():
-            if isinstance(value, UploadFile):
-                await value.aclose()
-
-
 class Headers(typing.Mapping[str, str]):
     """
     An immutable, case-insensitive multidict.
@@ -405,25 +310,29 @@ class Headers(typing.Mapping[str, str]):
     ) -> None:
         self._list = []  # type: typing.List[typing.Tuple[bytes, bytes]]
         if headers is not None:
-            assert raw is None, 'Cannot set both "headers" and "raw".'
-            assert scope is None, 'Cannot set both "headers" and "scope".'
             self._list = [
                 (key.lower().encode("latin-1"), value.encode("latin-1"))
                 for key, value in headers.items()
             ]
         elif raw is not None:
-            assert scope is None, 'Cannot set both "raw" and "scope".'
             self._list = raw
         elif scope is not None:
-            self._list = scope["headers"]
+            self._list = list(scope["headers"])
         elif environ is not None:
             self._list = [
-                (
-                    key.lower().replace("_", "-").encode("latin-1"),
-                    value.encode("latin-1"),
+                (key.lower().replace("_", "-").encode("latin-1"), value.encode("latin-1"))
+                for key, value in chain(
+                    (
+                        (key[5:], value)
+                        for key, value in environ.items()
+                        if key.startswith("HTTP_")
+                    ),
+                    (
+                        (key, value)
+                        for key, value in environ.items()
+                        if key in ("CONTENT_TYPE", "CONTENT_LENGTH")
+                    ),
                 )
-                for key, value in environ.items()
-                if key.startswith("HTTP_") or key in ("CONTENT_TYPE", "CONTENT_LENGTH")
             ]
 
     @property
@@ -438,8 +347,7 @@ class Headers(typing.Mapping[str, str]):
 
     def items(self) -> typing.List[typing.Tuple[str, str]]:  # type: ignore
         return [
-            (key.decode("latin-1"), value.decode("latin-1"))
-            for key, value in self._list
+            (key.decode("latin-1"), value.decode("latin-1")) for key, value in self._list
         ]
 
     def get(self, key: str, default: typing.Any = None) -> typing.Any:
