@@ -1,15 +1,21 @@
+import sys
 from types import TracebackType
 from typing import (
     Any,
-    Type,
-    Tuple,
+    Awaitable,
     Callable,
     Iterable,
     MutableMapping,
     Optional,
-    Awaitable,
+    Tuple,
+    Type,
 )
 
+if sys.version_info[:2] < (3, 8):
+    from typing_extensions import Final, Literal, TypedDict, final
+else:
+    from typing import Final, Literal, TypedDict, final
+
 __all__ = [
     "Scope",
     "Message",
@@ -20,6 +26,12 @@ __all__ = [
     "Environ",
     "StartResponse",
     "WSGIApp",
+] + [
+    # built-in types
+    "TypedDict",
+    "Literal",
+    "Final",
+    "final",
 ]
 
 # ASGI
